@@ -1,22 +1,24 @@
 from typing import Iterable, Tuple
 
 import numpy as np
-from pydts.examples_utils.simulations_data_config import *
+from .simulations_data_config import *
 from matplotlib import pyplot as plt
 import seaborn as sns
 from lifelines import KaplanMeierFitter
-from pydts.config import *
+from ..config import *
 import os
+import string
 import warnings
 warnings.filterwarnings('ignore')
 
 
+
 def add_panel_text(ax, text, xplace=-0.15, fsz=17):
     ax.text(xplace, 1.1, text, transform=ax.transAxes, fontsize=fsz,
             fontweight='bold', va='top', ha='right')
 
 
-def plot_first_model_coefs(models, times, expanded_train_df, n_cov=5, filename=None):
+def plot_first_model_coefs(models, times, train_df, n_cov=5, filename=None):
     fig, axes = plt.subplots(1, 2, figsize=(14, 6))
     ax = axes[0]
     ax.tick_params(axis='both', which='major', labelsize=15)
@@ -32,8 +34,8 @@ def plot_first_model_coefs(models, times, expanded_train_df, n_cov=5, filename=N
     ax.legend(loc='upper center', fontsize=14)
     ax.set_ylim([-3, 0.5])
     ax2 = ax.twinx()
-    ax2.hist(expanded_train_df['X'], color='r', alpha=0.3, bins=times)
-    ax2.set_ylabel('N patients', fontsize=16, color='red')
+    ax2.hist(train_df[train_df['J'] != 0]['X'], color='r', alpha=0.3, bins=times)
+    ax2.set_ylabel('N events', fontsize=16, color='red')
     ax2.tick_params(axis='y', colors='red')
 
     ax = axes[1]
@@ -76,7 +78,7 @@ def plot_second_model_coefs(alpha_df, beta_models, times, n_cov=5, filename=None
     ax.set_ylim([-3, 0.5])
     ax2 = ax.twinx()
     ax2.bar(alpha_df.groupby('X')['n_jt'].sum().index, alpha_df.groupby('X')['n_jt'].sum().values, color='r', alpha=0.3)
-    ax2.set_ylabel('N patients', fontsize=16, color='red')
+    ax2.set_ylabel('N events', fontsize=16, color='red')
     ax2.tick_params(axis='y', colors='red')
 
     ax = axes[1]
@@ -148,7 +150,7 @@ def plot_models_coefficients(alpha_dict: dict, beta_dict: dict, times: Iterable,
     ax.set_ylim([-3, 0.5])
     ax2 = ax.twinx()
     ax2.bar(times, njt_counts, color='r', alpha=0.3)
-    ax2.set_ylabel('N patients', fontsize=16, color='red')
+    ax2.set_ylabel('N events', fontsize=16, color='red')
     ax2.tick_params(axis='y', colors='red')
 
     ax = axes[1]
@@ -188,6 +190,7 @@ def plot_LOS_simulation_figure1(data_df):
     fig, axes = plt.subplots(2, 2, figsize=(14, 8))
 
     ax = axes[0, 0]
+    add_panel_text(ax=ax, text='a')
     data_df.groupby([ADMISSION_YEAR_COL, DEATH_MISSING_COL]).size().unstack().plot(kind='bar', stacked=True, ax=ax)
     ax.set_ylabel('Number of patients', fontsize=font_sz)
     ax.set_xlabel('Year of admission', fontsize=font_sz)
@@ -195,6 +198,7 @@ def plot_LOS_simulation_figure1(data_df):
     ax.set_ylim([0, 1000])
 
     ax = axes[0, 1]
+    add_panel_text(ax=ax, text='b')
     tmp = data_df[[AGE_COL, GENDER_COL]]
     tmp[AGE_COL] = pd.cut(tmp[AGE_COL], bins=AGE_BINS, labels=AGE_LABELS)
     tmp.groupby([AGE_COL, GENDER_COL]).size().unstack().plot(kind='bar', ax=ax)
@@ -203,6 +207,7 @@ def plot_LOS_simulation_figure1(data_df):
     ax.set_xticklabels(AGE_LABELS, rotation=90)
 
     ax = axes[1, 0]
+    add_panel_text(ax=ax, text='c')
     ser = data_df.sort_values(by=[PATIENT_NO_COL, ADMISSION_SERIAL_COL]).drop_duplicates(
         subset=[PATIENT_NO_COL], keep='last')[ADMISSION_SERIAL_COL]
     sns.distplot(ser, kde=False, ax=ax)
@@ -221,6 +226,7 @@ def plot_LOS_simulation_figure1(data_df):
     ax.grid('both', 'both')
 
     ax = axes[1, 1]
+    add_panel_text(ax=ax, text='d')
     kmf = KaplanMeierFitter(label='In-hospital death censoring')
     T = data_df[DISCHARGE_RELATIVE_COL].values
     E = data_df[DEATH_RELATIVE_COL].isnull().astype(int).values
@@ -234,7 +240,8 @@ def plot_LOS_simulation_figure1(data_df):
     kmf.plot_survival_function(ax=ax)
 
     ax.set_ylabel('Population', fontsize=font_sz)
-    ax.set_xlabel('Days from hospitalization', fontsize=font_sz)
+    ax.set_xlabel('LOS', fontsize=font_sz)
+    ax.set_xlim([0, 30])
     ax.grid()
 
     fig.tight_layout()
@@ -244,7 +251,7 @@ def plot_LOS_simulation_figure2(data_df):
     tmp['binned_age'] = pd.cut(tmp[AGE_COL], bins=AGE_BINS, labels=AGE_LABELS)
     tmp['death_at_hosp_ind'] = tmp[DEATH_RELATIVE_COL].notnull().astype(int)
 
-    max_time = 130
+    max_time = 30
     total_ad = len(data_df)
     ihd = (tmp.groupby(DEATH_RELATIVE_COL).size() / total_ad).reindex(range(0, max_time + 1)).fillna(0)
     ihd = ihd.loc[:max_time].cumsum()
@@ -253,25 +260,27 @@ def plot_LOS_simulation_figure2(data_df):
     released = released.loc[:max_time].cumsum()
     o = np.ones_like(released)
 
-    fig, axes = plt.subplots(3, 2, figsize=(12, 10))
+    fig, axes = plt.subplots(2, 2, figsize=(12, 7))
 
     text_sz = 12
     every_nth = 5
 
     ax = axes[0, 0]
+    add_panel_text(ax=ax, text='a')
     ax.fill_between(x=released.index, y1=(o - released.values - ihd.values), color='b', alpha=0.4)
     ax.fill_between(x=released.index, y1=o, y2=(o - released.values), color='g', alpha=0.4)
     ax.fill_between(x=released.index, y1=(o - released.values), y2=(o - released.values - ihd.values), color='r',
                     alpha=0.4)
-    ax.text(x=1, y=0.025, s='Hospitalized', fontsize=text_sz)
-    ax.text(x=max_time - 30, y=0.8, s='Released', fontsize=text_sz)
-    ax.text(x=max_time - 30, y=0.05, s='Dead', fontsize=text_sz)
+    ax.text(x=1, y=0.2, s='Hospitalized', fontsize=text_sz)
+    ax.text(x=20, y=0.8, s='Released', fontsize=text_sz)
+    ax.text(x=20, y=0.3, s='Dead', fontsize=text_sz)
     ax.set_ylim([0, 1])
     ax.set_xlim([0, max_time])
-    ax.set_xlabel('Days from hospitalization day', fontsize=font_sz)
+    ax.set_xlabel('LOS', fontsize=font_sz)
     ax.set_ylabel('Patient status ratio', fontsize=font_sz)
 
     ax = axes[0, 1]
+    add_panel_text(ax=ax, text='b')
     tmp[tmp['death_at_hosp_ind'] == 1].groupby(['binned_age', GENDER_COL]).size().unstack().plot(kind='bar', ax=ax)
     ax.set_xlabel('Age [years]', fontsize=font_sz)
     ax.set_ylabel('Number of patients', fontsize=font_sz)
@@ -279,12 +288,13 @@ def plot_LOS_simulation_figure2(data_df):
     ax.set_title('In-hospital Death', fontsize=title_sz)
 
     ax = axes[1, 0]
+    add_panel_text(ax=ax, text='c')
     ser = tmp[tmp['death_at_hosp_ind'] != 1].groupby(DISCHARGE_RELATIVE_COL).size()
     ser = ser.reindex(range(0, max_time + 1)).fillna(0)
     ser.plot(kind='bar', ax=ax)
     ax.set_xlim([0, max_time])
     ax.set_ylabel('Released', fontsize=font_sz)
-    ax.set_xlabel('Days from Hospitalization', fontsize=font_sz)
+    ax.set_xlabel('LOS', fontsize=font_sz)
     ax.grid(axis='y')
 
     for idl, label in enumerate(ax.xaxis.get_ticklabels()):
@@ -292,45 +302,48 @@ def plot_LOS_simulation_figure2(data_df):
             label.set_visible(False)
 
     ax = axes[1, 1]
+    add_panel_text(ax=ax, text='d')
     ser = tmp[tmp['death_at_hosp_ind'] == 1].groupby(DEATH_RELATIVE_COL).size()
     ser = ser.reindex(range(0, max_time + 1)).fillna(0)
     ser.plot(kind='bar', ax=ax)
     ax.set_xlim([0, max_time])
     ax.set_ylabel('Died', fontsize=font_sz)
-    ax.set_xlabel('Days from Hospitalization', fontsize=font_sz)
+    ax.set_xlabel('LOS', fontsize=font_sz)
     ax.grid(axis='y')
 
     for idl, label in enumerate(ax.xaxis.get_ticklabels()):
         if (idl % every_nth) > 0:
             label.set_visible(False)
 
-    ax = axes[2, 0]
-    ser = tmp[tmp['death_at_hosp_ind'] != 1].groupby(DISCHARGE_RELATIVE_COL).size()
-    ser = ser.reindex(range(0, max_time + 1)).fillna(0)
-    ser.plot(kind='bar', logy=True, ax=ax)
-    ax.set_xlim([0, max_time])
-    ax.set_ylabel('Released', fontsize=font_sz)
-    ax.set_xlabel('Days from Hospitalization', fontsize=font_sz)
-    ax.grid(axis='y')
-    ax.set_ylim([0, 1000])
-
-    for idl, label in enumerate(ax.xaxis.get_ticklabels()):
-        if (idl % every_nth) > 0:
-            label.set_visible(False)
-
-    ax = axes[2, 1]
-    ser = tmp[tmp['death_at_hosp_ind'] == 1].groupby(DEATH_RELATIVE_COL).size()
-    ser = ser.reindex(range(0, max_time + 1)).fillna(0)
-    ser.plot(kind='bar', logy=True, ax=ax)
-    ax.set_xlim([0, max_time])
-    ax.set_ylabel('Died', fontsize=font_sz)
-    ax.set_xlabel('Days from Hospitalization', fontsize=font_sz)
-    ax.grid(axis='y')
-    ax.set_ylim([0, 1000])
-
-    for idl, label in enumerate(ax.xaxis.get_ticklabels()):
-        if (idl % every_nth) > 0:
-            label.set_visible(False)
+    # ax = axes[2, 0]
+    # add_panel_text(ax=ax, text='e')
+    # ser = tmp[tmp['death_at_hosp_ind'] != 1].groupby(DISCHARGE_RELATIVE_COL).size()
+    # ser = ser.reindex(range(0, max_time + 1)).fillna(0)
+    # ser.plot(kind='bar', logy=True, ax=ax)
+    # ax.set_xlim([0, max_time])
+    # ax.set_ylabel('Released', fontsize=font_sz)
+    # ax.set_xlabel('Days from Hospitalization', fontsize=font_sz)
+    # ax.grid(axis='y')
+    # ax.set_ylim([0, 1000])
+    #
+    # for idl, label in enumerate(ax.xaxis.get_ticklabels()):
+    #     if (idl % every_nth) > 0:
+    #         label.set_visible(False)
+    #
+    # ax = axes[2, 1]
+    # add_panel_text(ax=ax, text='f')
+    # ser = tmp[tmp['death_at_hosp_ind'] == 1].groupby(DEATH_RELATIVE_COL).size()
+    # ser = ser.reindex(range(0, max_time + 1)).fillna(0)
+    # ser.plot(kind='bar', logy=True, ax=ax)
+    # ax.set_xlim([0, max_time])
+    # ax.set_ylabel('Died', fontsize=font_sz)
+    # ax.set_xlabel('Days from Hospitalization', fontsize=font_sz)
+    # ax.grid(axis='y')
+    # ax.set_ylim([0, 1000])
+    #
+    # for idl, label in enumerate(ax.xaxis.get_ticklabels()):
+    #     if (idl % every_nth) > 0:
+    #         label.set_visible(False)
 
     fig.tight_layout()
 
@@ -351,13 +364,15 @@ def plot_LOS_simulation_figure3(data_df):
 
 
 # todo: move from here
-def compare_beta_models_for_example(first_models: dict, second_models: dict, n_cov: int = 5) -> dict:
+def compare_beta_models_for_example(first_models: dict, second_models: dict,
+                                    n_cov: int = 5, real_coef_dict: dict = None) -> dict:
     """
 
     Args:
         first_models:
         second_models:
         n_cov:
+        real_coef_dict (dict):
 
     Returns:
 
@@ -367,16 +382,7 @@ def compare_beta_models_for_example(first_models: dict, second_models: dict, n_c
         "alpha": {},
         "beta": {}
     }
-    real_coef_dict = {
-        "alpha": {
-            1: lambda t: -1 - 0.3 * np.log(t),
-            2: lambda t: -1.75 - 0.15 * np.log(t)
-        },
-        "beta": {
-            1: -np.log([0.8, 3, 3, 2.5, 2]),
-            2: -np.log([1, 3, 4, 3, 2])
-        }
-    }
+    assert real_coef_dict is not None, "The user should supply the coefficients of the experiment"
     for event in first_models.keys():
         for model_type in models_dict.keys():
             if model_type == "alpha":
@@ -402,26 +408,29 @@ def compare_beta_models_for_example(first_models: dict, second_models: dict, n_c
     return models_dict
 
 
-def plot_reps_coef_std(rep_dict: dict, return_summary: bool = True):
+def plot_reps_coef_std(rep_dict: dict, return_summary: bool = True, filename: str = None):
     """
 
     Args:
         rep_dict:
         return_summary:
-
+        filename:
     Returns:
 
     """
     # todo: use mean as well
     # todo: split to different functions
-    fig, axes = plt.subplots(2, 2, figsize=(12, 10))
-    coef_types = list(rep_dict[0].keys())  # alpha, beta
-    event_types = rep_dict[0][coef_types[0]].keys()
+    alphabet_list = list(string.ascii_lowercase)
+    first_key = next(iter(rep_dict))    # deal with cases where there isn't 0 in samples
+    coef_types = list(rep_dict[first_key].keys())  # alpha, beta
+    event_types = rep_dict[first_key][coef_types[0]].keys()
     mapping = {t: i for i, t in enumerate(coef_types)}
+    fig, axes = plt.subplots(len(coef_types), len(event_types), figsize=(12, 10))
     res_dict = {coef: {event_type: None for event_type in event_types} for coef in coef_types}
-    for coef_type in coef_types:
-        for event_type in event_types:
+    for idct, coef_type in enumerate(coef_types):
+        for idet, event_type in enumerate(event_types):
             ax = axes[mapping[coef_type]][event_type - 1]
+            add_panel_text(ax=ax, text=alphabet_list[idct*len(event_types)+idet])
             ax.tick_params(axis='both', which='major', labelsize=15)
             ax.tick_params(axis='both', which='minor', labelsize=15)
             df = pd.concat([dfs[coef_type][event_type] for dfs in rep_dict.values()])
@@ -431,30 +440,52 @@ def plot_reps_coef_std(rep_dict: dict, return_summary: bool = True):
             temp_df.columns = temp_df.columns.get_level_values(0) + "_" + temp_df.columns.get_level_values(1)
             res_dict[coef_type][event_type] = temp_df.copy()
             temp_df.plot(x="Lee_std", y="Ours_std", kind="scatter", ax=ax)
-            ax.set_xlabel("Lee_std", fontsize=18)
-            ax.set_ylabel("Ours_std", fontsize=18)
+            ax.set_xlabel("Lee std", fontsize=18)
+            ax.set_ylabel("Ours std", fontsize=18)
             ax.plot([0, 1], [0, 1], "--", transform=ax.transAxes, alpha=0.3, color="tab:green");
             ax.grid()
+            # todo remove paper limits
+            if idct == 0:
+                ax.set_xlim([0, 0.6])
+                ax.set_ylim([0, 0.6])
+            elif ((idct == 1) and (idet == 0)):
+                ax.set_xlim([0.02, 0.04])
+                ax.set_ylim([0.02, 0.04])
+            elif ((idct == 1) and (idet == 1)):
+                ax.set_xlim([0.04, 0.06])
+                ax.set_ylim([0.04, 0.06])
             latter = "\\alpha" if coef_type == "alpha" else "\\beta"
             ax.set_title(f"${latter}{event_type}$", fontsize=18)
     fig.tight_layout()
     fig.show()
+    if filename is not None:
+        fig.savefig(os.path.join(OUTPUT_DIR, filename), dpi=300)
+
     if return_summary:
         return res_dict
 
 
 def plot_times(times_dict: dict,
-               filename: str = None) -> None:
+               filename: str = None,
+               ax = None, color='tab:blue') -> None:
     """
 
     Args:
         times_dict:
-
+        filename:
     Returns:
 
     """
-    ax = pd.DataFrame.from_dict(times_dict).boxplot(figsize=(8, 6), boxprops={"lw": 1.5, "color": "tab:blue"},
-                                                    medianprops={"lw": 2, "color": "tab:green"})
+
+    if ax is None:
+
+        ax = pd.DataFrame.from_dict(times_dict).boxplot(figsize=(8, 6), boxprops={"lw": 1.5, "color": "tab:blue"},
+                                                        medianprops={"lw": 2, "color": "tab:green"},
+                                                        flierprops={'markeredgecolor': "tab:blue"})
+    else:
+        pd.DataFrame.from_dict(times_dict).boxplot(boxprops={"lw": 1.5, "color": color},
+                                                   flierprops={'markeredgecolor': color},
+                                                   medianprops={"lw": 2, "color": color}, ax=ax)
 
     # ax.set_ylim(0, 15)
     ax.tick_params(axis='both', which='major', labelsize=15)
@@ -467,6 +498,7 @@ def plot_times(times_dict: dict,
     plt.show()
     if filename is not None:
         ax.figure.savefig(os.path.join(OUTPUT_DIR, filename), dpi=300)
+    return ax
 
 
 def plot_cif_plots(pred_df: pd.DataFrame, event: str, return_ax: bool = False, ax: plt.Axes = None,
@@ -485,8 +517,6 @@ def plot_cif_plots(pred_df: pd.DataFrame, event: str, return_ax: bool = False, a
     Returns:
 
     """
-    import matplotlib.ticker as mtick
-
     cif_cols = pred_df.columns[pred_df.columns.str.startswith("cif")]
 
     event_cif_cols = cif_cols[cif_cols.str.contains(f"j{event}")]
@@ -500,9 +530,8 @@ def plot_cif_plots(pred_df: pd.DataFrame, event: str, return_ax: bool = False, a
     ax.tick_params(axis='both', which='minor', labelsize=15)
     ax.set_xticks(event_x)
     ax.set_xticklabels(event_x)
-    # ax.yaxis.set_major_formatter(mtick.PercentFormatter(1.0))
     y_min, y_max = get_y_perc_limits(pred_df, event_cif_cols, pad=pad, scale=scale)
-    ax.set_ylim([y_min, y_max])
+    ax.set_ylim(bottom=y_min, top=y_max)
     ax.set_xlabel("t", fontdict={'size': 18})  # , "weight": "bold"
     ax.set_ylabel(f"CIF of $J_{event}$", fontdict={'size': 18})  # , "weight": "bold"
     ax.grid()
